
#include<iostream>
#include<sstream>
#include<string>
#include<vector>
#include<map>
using namespace std;



map<int, map<char, int> > CreateAutomaton(string str)
{
    map<int, map<char, int> > delta;
    vector<bool> current(str.size()+1,false);
    current[0] = true;
    vector<bool> next(str.size()+1,false);
    next[0] = true;

    for(int i = 0; i < str.size(); i++)
    {
        // make transitions for state i (depending on which states the NFA would
        // be in after reading w := str[0]..str[i-1]
        for(int j = 0; j < current.size(); j++)
        {
            if(current[j])
            {
                // current[j] == true  iff  the NFA can be in j after reading w
                delta[i][str[j]] = j+1;
                if(str[j] == str[i])
                    next[j+1] = true;
            }
        }
        current = next;
        next = vector<bool>(str.size(),false);
        next[0] = true;
    }
    return delta;
}

void AutomatonExtract(string startstring, string endstring, istream& is, ostream& os)
{
    map<int, map<char,int> > automaton1 = CreateAutomaton(startstring);
    map<int, map<char,int> > automaton2 = CreateAutomaton(endstring);
    
    int nextstate = 0;
    int state = 0;
    char readchar;
    while(true)
    {
        state = 0;
        while(true)
        {
            // Automaton 1
            if(is.peek() == -1)
                return;
            readchar = is.get();

            if(automaton1[state].find(readchar) == automaton1[state].end())
            {
                state = 0;
            }
            else
            {
                state = automaton1[state][readchar];
                if(state == automaton1.size())
                    break;
            }
        }

        state = 0;
        while(true)
        {
            // Automaton 2
            if(is.peek() == -1)
            {
                for(int i = 0; i < state; i++)
                    os << endstring[i];
                os << endl;
                return;
            }
            
            readchar = is.get();
        
            if(automaton2[state].find(readchar) == automaton2[state].end())
                nextstate = 0;
            else
                nextstate = automaton2[state][readchar];


            if(nextstate <= state) // part of the buffer is discarded -> print it, as it was not part of the terminating-string
            {
                for(int i = nextstate; i < state; i++)
                    os << endstring[i];
                os << readchar;
            }
            
            state = nextstate;
            if(state == automaton2.size())
            {
                os << endl;
                break;
            }
        }
        
    }
}

int main(int argc, char** argv)
{
    if(argc <= 2)
    {
        cerr << "usage: " << argv[0] << " <starttag> <endtag>\n";
        cerr << "       the program copies all portions between <starttag> and <endtag>\n";
        cerr << "       from stdin to stdout.\n";
        cerr << "       example: cat foo.html | " << argv[0] << " 'href=\"' '\"'\n";
        cerr << "       would extract the targets of hyperlinks.\n";
        return 1;
    }

    AutomatonExtract(argv[1], argv[2], cin, cout);

    return 0;
}
